/*
 * file: window_control.c
 * This file is part of the libpv library examples.
 *
 * Copyright 2005 Pengutronix
 * Juergen Beisert j.beisert@pengutronix.de
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 *
 * PURPOSE:
 * Interacts with window_lift (the simulator). This part
 * controls the lifter. It waits for buttons (up, down), starts
 * the motor and stops it when the buttons hit again or
 * the window reaches its end position.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <signal.h>

#include <libpv.h>

static int8_t *button_up;
static int8_t *button_down;
static uint32_t *photo_sensor;
static int8_t *end_switch;
static int8_t *motor_up;
static int8_t *motor_down;

static int	loop=1;

/**
 * to stop the endless loops
 * FIXME: Find another way to stop this demo
 * regulary
 **/
void signalhandler (int sig)
{
	if (sig == SIGTERM || sig == SIGINT)
		loop=0;	/* end the loop in main() */
}

int main(int argc,char *argv[])
{
	pv_engine_attr_t	attr;
	pv_engine_t			pv_engine;
	void				*var;

	if (argc > 1) {
		/*
		* parse the given parameters
		*/
		if (pv_engine_attr_init_from_cmdline(&attr, argc, argv))
			return(EXIT_FAILURE);
	}
	else {
		/*
		 * Without any parameters setup default values
		 */
		pv_engine_attr_init(&attr);
		strcpy(attr.desc_filename,"window_lift.xml");
		strcpy(attr.store_filename,"window_lift.bin");
		attr.jvisu_port=0;
	}

	/*
	 * establish connection to the process image
	 */
	if (pv_init(&pv_engine, &attr))
		return(EXIT_FAILURE);

	/*
	 * Register our variables
	 */
	if ((button_up=pv_register_int8(&pv_engine,"button1")) == NULL) {
		printf("Cannot register button_up\n");
		goto exit_1;
	}
	if ((button_down=pv_register_int8(&pv_engine,"button2")) == NULL) {
		printf("Cannot register button_down\n");
		goto exit_1;
	}
	if ((photo_sensor=pv_register_uint32(&pv_engine,"timestamp")) == NULL) {
		printf("Cannot register photo_sensor\n");
		goto exit_1;
	}
	if ((end_switch=pv_register_int8(&pv_engine,"button3")) == NULL) {
		printf("Cannot register end_switch\n");
		goto exit_1;
	}
	if ((motor_up=pv_register_int8(&pv_engine,"motor1")) == NULL) {
		printf("Cannot register motor_up\n");
		goto exit_1;
	}
	if ((motor_down=pv_register_int8(&pv_engine,"motor2")) == NULL) {
		printf("Cannot register motor_down\n");
		goto exit_1;
	}

	/*
	 * What we are doing here:
	 * - Wait for buttons, sensors and switchs
	 * - Control the motor
	 */
	pv_event_register(&pv_engine,button_up);
	pv_event_register(&pv_engine,button_down);
	pv_event_register(&pv_engine,photo_sensor);	/* notifies movement */
	pv_event_register(&pv_engine,end_switch);	/* end position of window */

/*	signal(SIGTERM, signalhandler);
	signal(SIGINT, signalhandler); */

	pv_lock(&pv_engine);
	*motor_up=0;
	*motor_down=0;
	pv_event_generate(&pv_engine,motor_up);
	pv_event_generate(&pv_engine,motor_down);
	pv_unlock(&pv_engine);

	while (loop) {
		pv_event_await_syncronous(&pv_engine,&var);
		pv_lock(&pv_engine);

		if (var == button_up) {
			if (*button_up != 0) {
				*motor_up=1;	/* start motor */
				pv_event_generate(&pv_engine,motor_up);
				printf("brrr...");	/* motor's up sound ;-) */
			}
			else {
				if (*motor_up != 0) {
					*motor_up=0;	/* stop motor */
					pv_event_generate(&pv_engine,motor_up);
				}
			}
		}

		if (var == button_down) {
			if (*button_down != 0) {
				*motor_down=1;	/* start motor */
				pv_event_generate(&pv_engine,motor_down);
				printf("frrr...");	/* motor's down sound ;-) */
			}
			else {
				if (*motor_down != 0) {
					*motor_down=0;	/* stop motor */
					pv_event_generate(&pv_engine,motor_down);
				}
			}
		}

		if (var == photo_sensor) {	/* show movement */
			if (*photo_sensor != 0)
				printf("tik...");
			else
				printf("tak...");
		}

		if (var == end_switch) {
			if (*end_switch != 0) {
				printf("pock\n");	/* sound of window bangs end position */
				if (*motor_up != 0) {
					*motor_up=0;	/* stop motor */
					pv_event_generate(&pv_engine,motor_up);
				}
				else {
					*motor_down=0;	/* stop motor */
					pv_event_generate(&pv_engine,motor_down);
				}
			}
		}
		pv_unlock(&pv_engine);
		fflush(stdout);
	}
exit_1:
	pv_destroy(&pv_engine);
	return(EXIT_SUCCESS);
}

/*
#
# vim settings
#
# vim:set ts=4:
# vim:set shiftwidth=4:
# vim:set nowrap:
#
*/

/* end of file window_control.c */
